from datetime import datetime
from sqlalchemy import Column, Integer, String, Boolean, DateTime
from sqlalchemy.orm import relationship
from werkzeug.security import generate_password_hash, check_password_hash
from models.base import Base, Session

class User(Base):
    __tablename__ = 'users'
    
    id = Column(Integer, primary_key=True)
    username = Column(String(50), unique=True, nullable=False)
    email = Column(String(120), unique=True, nullable=False)
    password_hash = Column(String(256), nullable=False)
    full_name = Column(String(100))
    role = Column(String(20), default='user')  # 'admin' or 'user'
    image_url = Column(String(255))
    is_active = Column(Boolean, default=True)
    created_at = Column(DateTime, default=datetime.utcnow)
    last_login = Column(DateTime)

    @classmethod
    def get_by_id(cls, user_id):
        session = Session()
        try:
            return session.query(cls).filter(cls.id == user_id).first()
        finally:
            session.close()

    @classmethod
    def get_by_username(cls, username):
        session = Session()
        try:
            return session.query(cls).filter(cls.username == username).first()
        finally:
            session.close()

    @classmethod
    def get_by_email(cls, email):
        session = Session()
        try:
            return session.query(cls).filter(cls.email == email).first()
        finally:
            session.close()

    def set_password(self, password):
        self.password_hash = generate_password_hash(password)

    def check_password(self, password):
        return check_password_hash(self.password_hash, password)

    def update_last_login(self):
        session = Session()
        try:
            self.last_login = datetime.utcnow()
            session.merge(self)
            session.commit()
        finally:
            session.close()
